- [TL;DR](#org9e79972)
- [Topology](#org5acf069)
- [Algorithms](#org633aad7)
- [Examples](#orgf3a71cb)
- [Batch mode](#orgcf0c55e)
  - [Running batch simulations with Built-in daemons](#org8a1d24b)
  - [Running batch simulations with manual daemons](#orgb210f36)
  - [Running batch simulations with `lurette`](#org3054f33)
  - [Viewing Results](#org5dd2887)
  - [The `sasa` CLI](#orgf675f37)
- [Interactive mode](#org8faa1ed)
  - [Example: use `rdbg` from the `test/alea-coloring/` directory](#org3059bee)
  - [The exemples of test directory](#orgae03a82)
  - [Running interactive sessions with `rdbg`](#orga8a36c0)
  - [Getting `rdbg` on-line help](#org364f2f1)
  - [Useful Modules](#orga121e4d)
- [Install](#org4f267cb)
  - [Via opam (version >= 2.\*)](#org5050542)
  - [Via docker](#org8ac8d84)
  - [From source](#org2ca00f9)
- [More](#org08e4f38)
- [FAQ](#org92180aa)
  - [Is there a FAQ?](#org6a39565)
  - [I have a compilation error that I don't understand](#org98a195b)
  - [I have the error `Invalid_argument("compare: functional value")`](#org270d83f)



<a id="org9e79972"></a>

# TL;DR

<a id="org36389ee"></a> SASA is a **Self-stabilizing Algorithms SimulAtor**, based on the so-called **Atomic State model** (ASM) introduced by <span class="underline">Dijkstra</span> in its seminal article on [Self-stabilizing distributed algorithms](http://www.cs.utexas.edu/~EWD/ewd04xx/EWD426.PDF).

Basically, one needs to provide:

1.  a topology, made of nodes and transitions (via a [dot](https://en.wikipedia.org/wiki/DOT_(graph_description_language)) file)
2.  the algorithms attached to nodes (via [`ocaml`](https://ocaml.org/) programs)

The fastest way to get started is to copy the files provided in the `test/skeleton` directory, and to modify them:

```sh
cd test
cp -rf skeleton my_algo
cd my_algo
make ring.cmxs
sasa ring.dot
```

SASA source code: <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa>

Some Tutorials: <https://verimag.gricad-pages.univ-grenoble-alpes.fr/vtt/tags/sasa/>

[![img](poster.png)](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/sasa/poster.pdf)


<a id="org5acf069"></a>

# Topology

<a id="org88651ee"></a> <a id="orgc302762"></a> The topology is given via `.dot` files, that should

1.  follow the [graphviz/dot format](https://en.wikipedia.org/wiki/DOT_(graph_description_language))
2.  have nodes **labelled** by the `algo` field

```dot
graph ring {
 p1 [algo="some_algo.ml"]
 p2 [algo="some_algo.ml"]
 p3 [algo="some_algo.ml"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml"]
 p6 [algo="some_algo.ml"]
 p7 [algo="some_algo.ml"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Of course the `some_algo.ml` file must exist.

In order to define initial state values to nodes, one can use `init` node [attribute](http://www.graphviz.org/doc/info/attrs.html).

```dot
graph ring {
 p1 [algo="some_algo.ml" init="i=1"]
 p2 [algo="some_algo.ml" init="i=2"]
 p3 [algo="some_algo.ml" init="i=42"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml=1"]
 p6 [algo="some_algo.ml=1"]
 p7 [algo="some_algo.ml=1"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

One can use graph [attributes](http://www.graphviz.org/doc/info/attrs.html) to set parameters that can be used by all nodes during the simulation.

```dot
graph ring {
 graph [diameter="4" some_other_global_parameter="42"]
 p1 [algo="some_algo.ml" init="i=1"]
 p2 [algo="some_algo.ml" init="i=2"]
 p3 [algo="some_algo.ml" init="i=42"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml=1"]
 p6 [algo="some_algo.ml=1"]
 p7 [algo="some_algo.ml=1"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Such parameters can be retrieved in Algorithms using the `Algo.get_graph_attribute : string -> string` function.

nb: a `Algo.diameter: unit -> int` function is provided, but it can be expensive to use.

Some tools are provided in the `sasa` [7](#org528f4e8) to generate such kinds of `dot` graphs: <https://verimag.gricad-pages.univ-grenoble-alpes.fr/vtt/articles/sasa-gg/>


<a id="org633aad7"></a>

# Algorithms

<a id="org45ccdff"></a>

The following has been generated from [algo.mli](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/blob/master/lib/algo/algo.mli)

<div class="html">
<iframe title="The Algo API" name="algo-api" width="700" height="700" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/algo/Algo/index.html>" alt="<sub>html</sub>/algo/Algo/index.html";> </iframe>

</div>


<a id="orgf3a71cb"></a>

# Examples

The [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory contains various examples of self-stabilizing distributed programs taken from the book \`\`Introduction to Distributed Self-Stabilizing Algorithms'' By Altisen, Devismes, Dubois, and Petit.

1.  `test/skeleton/`: a fake algorithm meant to be used as a skeleton
2.  `test/dijkstra-ring/`: Dijkstra token ring
3.  `test/unison/`: Synchronous unison
4.  `test/async-unison/`: Asynchronous unison
5.  `test/coloring/`: a graph coloring algorithm
6.  `test/alea-coloring/`: a randomized variant of the previous one
7.  `test/bfs-spanning-tree/`: a Breadth First Search Spanning tree construction

It also contains implementations of algorithms found in the literature:

1.  `test/st-CYH91`: another Spanning tree construction ("A self-stabilizing algorithm for constructing spanning trees" by Chen, Yu, and Huang in 1991)
2.  `test/bfs-st-HC92`: another BFS Spanning tree construction ("A self-stabilizing algorithm for constructing breadth-first trees" by Huang and Chen in 1992)
3.  `test/st-KK06_algo1` and `test/st-KK06_algo2`: another Spanning tree construction ("A Self-stabilizing Algorithm for Finding a Spanning Tree in a Polynomial Number of Moves" by Kosowski and Kuszner, 2006)
4.  `test/dfs/`: a Depth First Search using arrays (the \`\`atomic state model'' version of a [Depth First Search algorithm proposed by Collin and Dolev in 1994](http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.57.1100&rep=rep1&type=pdf))
5.  `test/dfs-list/`: the same Depth First Search, but using lists instead or arrays

Each directories contains working examples, which are checked using the Continuous Integration facilities of Gitlab (cf the [.gitlab-ci.yml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/.gitlab-ci.yml) script and the [CI pipelines](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/pipelines) results).

If you want to reproduce or understand what those non-regression tests do, please look at the `test/*/Makefile`, present in each directory (which all include the [test/Makefile.inc](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/Makefile.inc) one).

The `test` directory also contains sub-directories which gathers programs shared by all examples:

-   `test/lustre/`: contains Lustre programs used as (`test/*/*.lus`) oracles
-   `test/rdbg-utils/`: contains `ocaml` functions that can be used from `rdbg`
-   `test/my-rdbg-tuning.ml`: contains useful shortcuts/commands that can be used from `rdbg`.
-   `test/*/my-rdbg-tuning.ml`: includes `test/my-rdbg-tuning.ml` and defines commands specific to the example of the directory. Indeed, `rdbg`, once launched, first tries to read the content of the file name `my-rdbg-tuning.ml` (if it exists).


<a id="orgcf0c55e"></a>

# Batch mode

![img](./sasabatch.svg)


<a id="org8a1d24b"></a>

## Running batch simulations with Built-in daemons

To launch batch simulations, one can use the `sasa` Command Line Interface.

For instance, in order to simulate an algorithm defined in the `ring.dot` (cf [test/coloring](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/coloring), you can launch the following command:

```sh
sasa ring.dot 
```

By default, the distributed daemon will be used to activate actions. Several other daemon can be used, via one of the following options:

```sh
sasa -h | grep "\-daemon"
```

    --synchronous-daemon, -sd
    --central-daemon, -cd
    --locally-central-daemon, -lcd
    --distributed-daemon, -dd
    --custom-daemon, -custd


<a id="orgb210f36"></a>

## Running batch simulations with manual daemons

By using the `--custom-daemon` option (or `-custd` for short), you can play the role of the daemon. More precisely, you will be prompted for stating which actions should be activated, among the set of enabled actions.

```sh
cd ../../test/unison
make ring.cmxs
sasa ring.dot --custom-daemon
```

In the custom daemon mode, the daemon is executed outside `sasa`, by a process that communicates via the standard input/output using [RIF](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-5) conventions. More precisely, `sasa` writes on `stdout` a Boolean for each action and each process, that states if the corresponding action is enabled for the current step. Then it reads on `stdin` a Boolean for each action and each process that tells which actions (among the enabled ones) should be activated.

The daemon can thus be played by users that read and enter Boolean values.

In order to enter such input more easily, one can use [`luciole-rif`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-8)

```sh
luciole-rif sasa ring.dot --custom-daemon
```

It can also by simulated by [`lutin`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-4) programs via the use of [`lurette`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-9) (for batch executions) or [`rdbg`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-10) (for interactive sessions).

Built-in daemons can of course be programmed in Lutin. One can generate such daemons using the `--gen-lutin-daemon` option: `sasa --gen-lutin-daemon a_graph.dot`. It can be handy at least to get the daemons variables names in the good order if one to write its own daemon.


<a id="org3054f33"></a>

## Running batch simulations with `lurette`

If you want to perform simulations with custom daemons, you need to use `lurette`. For instance, in order to simulate an algorithm defined in the `ring.dot` (cf [test/coloring](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/coloring)) using a Lutin daemon defined in `ring.lut`, you can launch the following command:

```sh
lurette \
     -env "sasa ring.dot -custd" \
     -sut "lutin ring.lut -n distributed"
```

For `lurette`, the role of the SUT and the one of the environment is dual: the outputs of the SUT are the inputs of the environment, and vice-versa. The only difference is that the environment plays first. But `sasa` needs to play first, to be able to state which actions are enabled at the very first step. Hence `sasa` is used as a `lurette` environment, and the daemon program is used a `lurette` SUT.

If one wants to use an internal sasa daemon, no environment is needed and sasa can be used with `-sut`. In order to use an oracle defined in Lustre, one can use `lurette` as follows:

```sh
lurette \
     -sut "sasa ring.dot" \
     -oracle "lv6 ring_oracle.lus -n oracle -exec"
```


<a id="org5dd2887"></a>

## Viewing Results

`sasa -rif` and `lurette` generates `.rif` files that can be viewed with `gnuplot-rif` or `sim2chro`; cf <http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/reactive-toolbox/>


<a id="orgf675f37"></a>

## The `sasa` CLI

```sh
sasa --help
```

    usage: sasa [<option>]* <topology>.dot 
    use -h to see the available options.
    
    --synchronous-daemon, -sd
                Use a Synchronous daemon
    --central-daemon, -cd
                Use a Central daemon (selects exactly one action)
    --locally-central-daemon, -lcd
                Use a Locally Central daemon
                (i.e., never activates two neighbors actions in the same step)
    --distributed-daemon, -dd
                Use a Distributed daemon (which select at least one action). This is the default daemon.
    --custom-daemon, -custd
                Use a Custom daemon (forces --rif)
    --seed, -seed
                Set the pseudo-random generator seed of build-in daemons (wins over --replay)
    --replay, -replay
                Use the last generated seed to replay the last run
    --gen-register, -reg
                Generates the registering file and exit. 
    --length, -l <int>
                Maximum number of steps to be done (10000 by default).
    
    --version, -version, -v
                Display the sasa version and exit.
    --verbose, -vl <int>
                Set the verbose level
    --help, -help, -h
                Display main options
    --more, -m  Display more options

More `sasa` options:

```sh
sasa --more
```

    --rif, -rif Display only outputs on stdout (i.e., behave as a rif input file)
    --no-data-file, -nd
                Do not generate any data file
    --gen-lutin-daemon, -gld
                Generate Lutin daemons and exit
    --gen-lustre-oracle-skeleton, -glos
                Generate a Lustre oracle skeleton
    --list-algos, -algo
                Output the algo files used in the dot file and exit. 
    --dummy-input
                Add a dummy input to sasa so that built-in daemon can be used from rdbg
    --ignore-first-inputs, -ifi
                Ignore first inputs (necessary to use luciole via 
                lurette/rdbg/luciole-rif)
    --ocaml-version
                Display the version ocaml version sasa was compiled with and exit.


<a id="org8faa1ed"></a>

# Interactive mode

<a id="orgfa04b7c"></a> If you want to perform interactive session, you can launch `sasa` under the control of [rdbg](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/rdbg/). Before reading this section, please read at least the [Basic usage Section if the rdbg documentation](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/rdbg/README.html#Level0).


<a id="org3059bee"></a>

## Example: use `rdbg` from the `test/alea-coloring/` directory

All the sub-directories of the `test` directory are organized similarly. Let's have a look at the `test/alea-coloring/` directory. It contains the following files:

-   `my-rdbg-tuning.ml` that just includes [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) (some `my-rdbg-tuning.ml` files define more commands, such as the one in [test/async-unison/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/async-unison/my-rdbg-tuning.ml)).
-   `ring.dot`: a possible topology
-   `p.ml`: an algorithm used in `ring.dot`
-   `Makefile`: a Makefile that daemonstrates various ways of using `sasa`. In particular, it contains a rule named `rdbg`.

Try for instance:

```sh
cd test/coloring
make rdbg
```

This make rule (defined in `Makefile` and `../Makefile.inc`)

1.  generates the `ring.ml` file, that contains the registration code (that can indeed be generated by sasa, as explained in `algo.mli`).
2.  launches `rdbg` with some arguments. `rdbg` then prompts the user to enter one of the folling commands:

```sh
rdbg -o ring.rif -sut "$(sasa)  ring.dot --locally-central-daemon" 

[] create a fresh session
[q] quit
[s] do nothing

[/q/s]: 
```

By default, the first in the list is executed; hence if you press `[Return]` without any command, a fresh session is created. More precisely it creates a `rdbg_session.ml` file, that loads all the necessary modules to run an interactive session with `rdbg`, where `rdbg` simulates ring.dot with `sasa` using a locally distributed daemon.

For instance, you can type `nr[return]`, a `rdbg` command defined in [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) that steps until the next round is reached. Then, by typing `[return]`; `rdbg` will run the lastly used command, which is `nr`, which allow you to move forward in the execution from rounds to rounds. If you want to go backwards to the previous round, you can use the `br` command=, also defined in [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml).

nb: the next time `rdbg` is launched (without argument), you will be prompted with the possibility to reuse this session.

```sh
$ rdbg

[] #use "rdbg-session.ml"
[c] create a fresh session
[q] quit
[s] do nothing

[/c/q/s]: 
```

Typing `[Enter]` will therefore also load the `rdbg_session.ml` file we have just been using.


<a id="orgae03a82"></a>

## The exemples of test directory

The [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory contains several files and directories that take advantage of the versatility (programmability) of `rdbg` to perform interactive `sasa` simulations:

-   `test/rdbg-utils/`: contains `ocaml` functions that can be used from `rdbg`
-   `test/my-rdbg-tuning.ml`: contains useful shortcuts/commands that can be used from `rdbg`.
-   `test/*/my-rdbg-tuning.ml`: includes `test/my-rdbg-tuning.ml` and defines commands specific to the example of the directory. Indeed, `rdbg`, once launched, first tries to read the content of the file name `my-rdbg-tuning.ml` (it it exists).


<a id="orga8a36c0"></a>

## Running interactive sessions with `rdbg`

1.  type `rdbg`
2.  press enter to load the defaut session (`rdbg-session.ml`). Then you can type:
3.  `d` for displaying a (dynamic) dot graph with `dot`
4.  or ALTERNATIVELY `ne` to use the `neato` layout engine (`tw`, `ci`, `fd`, `sf`, `pa`, `os` to use the `twopi`, `circo`, `fdp`, `sfd`, `patchwork`, `osage` layout engines respectively)
5.  `s` to move one step forward
6.  `sd` to move one step forward and call `d` (to update the graph display [])
7.  `nr` to move one round forward
8.  `pr` to move one round backward
9.  `go` to move forward until convergence

All those commands are defined in (the common) [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) that is included in (locals) `test/*/my-rdbg-tuning.ml` that are included in (generated) `test/*/rdbg-session.ml` files. `my-rdbg-tuning.ml` contains straigthforward `ocaml` code that defines various `rdbg` shortcuts to ease the simulation of `sasa` systems. Feel free to tailor those command to yours needs by modyfying the local `my-rdbg-tuning.ml`!


<a id="org364f2f1"></a>

## Getting `rdbg` on-line help

Here are 2 useful entry-points to rdbg on-line help:

1.  `rdbg --help` and
2.  at the `rdbg` prompt, you can use the `l` command:

```sh
(rdbg) l
```

    82c2e944ce320891a508ab2190c82cf3


<a id="orga121e4d"></a>

## Useful Modules

Some modules, used by the sasa core engine, can be useful from `rdbg`.

<div class="html">
<iframe title="The Topology API" name="topology-api" width="700" height="500" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Topology/index.html>" alt="<./_html/sasacore/Sasacore/Topology/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The Diameter API" name="diameter-api" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Diameter/index.html>" alt="<./_html/sasacore/Sasacore/Diameter/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The Process API" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Process/index.html>" alt="<./_html/sasacore/Sasacore/Process/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The StringOf API" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/StringOf/index.html>" alt="<./_html/sasacore/Sasacore/StringOf/index.html>";> </iframe>

</div>


<a id="org4f267cb"></a>

# Install

<a id="org528f4e8"></a> <a id="org572cc9c"></a> In any case, you need to install the [graphviz](https://en.wikipedia.org/wiki/DOT_(graph_description_language)) tools suite and a pdf viewer that is able to auto-refresh (for instance `zathura`).


<a id="org5050542"></a>

## Via opam (version >= 2.\*)

One way to install `sasa` is via [opam](http://opam.ocaml.org/doc/Install.html), a package manager for `ocaml` programs. `opam` works out of the box with most Linux distributions and OSX (mac). `opam` ougth to work [on windows too](http://protz.github.io/ocaml-installer/).

Once opam (version >= 2.\*!) is installed and configured:

```sh
opam switch create 4.09.0
eval $(opam env)
opam repo add -a verimag-sync-repo "http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/opam-repository"
opam update -y
opam install -y sasa
```

nb: the `opam switch create 4.09.0`, which installs an `4.09.0` version of `ocaml`, should not be necessary in theory. If you don't do that, `opam` uses the `ocaml` of your system distribution. But in practice, `opam` tends to work better when working with one on its own switches.

If you want to be able to defined daemons in Lutin, or test oracles in Lustre, you can also install the following packages.

```sh
opam depext  -y lutin lustre-v6 
opam install -y lutin lustre-v6 
```

Once this is done, upgrading to the last version of the tools is as simple as:

```yaml
opam update
opam upgrade
```

In order to be able to use `luciole` and `sim2chro`, you also need to install the Lustre V4 distribution: <http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v4/distrib/index.html>


<a id="org8ac8d84"></a>

## Via docker

cf the Docker Install section of the [Synchrone Reactive Tool Box](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#docker).


<a id="org2ca00f9"></a>

## From source

You will need:

-   `make` (gnu)
-   a set of tools installable via [opam](https://opam.ocaml.org/)
    -   `dune`
    -   `ocamlgraph`
    -   `rdbg`
    -   `lutin` (not for compiling actually, but for using sasa with custom daemons)

```sh
opam install dune ocamlgraph rdbg lutin
```

One can mimick the content of the `test` job in the project [.gitlab-ci.yml Gitlab CI script](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/.gitlab-ci.yml).


<a id="org08e4f38"></a>

# More

-   Releases Notes: <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/releases>
-   Sources: <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa>
-   Tutorials: <https://verimag.gricad-pages.univ-grenoble-alpes.fr/vtt//categories/sasa/>


<a id="org92180aa"></a>

# FAQ


<a id="org6a39565"></a>

## Is there a FAQ?

Yes.

Beside, some tutorials are also available here: <https://verimag.gricad-pages.univ-grenoble-alpes.fr/vtt/tags/sasa/>


<a id="org98a195b"></a>

## I have a compilation error that I don't understand

-   Look at recent `.log` files (e.g., `rdbg.log`); they sometimes contain more information than what is printed onto the screen.
-   Do a `make clean`
-   Read carefully the error message. Sometimes it helps.
-   If the message is totally useless, please feel free to add an issue here <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/issues>


<a id="org270d83f"></a>

## I have the error `Invalid_argument("compare: functional value")`

Most probably you a try to compare 2 `'s Algo.neighbor`. It's an abstract type, hence you cannot compare them, i.e., you cannot use `Stdlib.compare`, nor its twins (`<>`, `>`, `<`, etc.), nor functions that use them (`min`, `max`, `List.sort`, etc.). You should compare their `pid` instead (if the network is not anonymous).
